@@ -13,12 +13,19 @@ from anasymod.fpga_boards.boards import *
 from anasymod.base_config import BaseConfig
 
 class EmuConfig:
-    def __init__(self, root, cfg_file, build_root=None):
+    def __init__(self, root, cfg_file, active_target, build_root=None):
+
+        self._valid_ila_values = [1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072]
 
         self.root = root
 
         # define and create build root
-        self.build_root = build_root if build_root is not None else os.path.join(root, 'build')
+        self.build_root_base = build_root if build_root is not None else os.path.join(root, 'build')
+
+        # define build root for functional models
+        self.build_root_functional_models = os.path.join(self.build_root_base, 'models')
+
+        self.build_root = os.path.join(self.build_root_base, active_target)
         if not os.path.exists(self.build_root):
             mkdir_p(self.build_root)
 
@@ -63,7 +70,16 @@ class EmuConfig:
 
     @property
     def ila_depth(self):
-        return 4096
+        if self.cfg.ila_depth in self._valid_ila_values:
+            return self.cfg.ila_depth
+        else:
+            raise Exception(f'Provided value for ILA depth is not valid, allows values are:{self.cfg.ila_depth}, '
+                            f'given: {self.cfg.ila_depth}')
+
+    def _update_build_root(self, active_target):
+        self.build_root = os.path.join(self.build_root_base, active_target)
+        if not os.path.exists(self.build_root):
+            mkdir_p(self.build_root)
 
     def _fetch_board(self):
         """
@@ -82,6 +98,8 @@ class EmuConfig:
             return TE0720()
         elif self.cfg.board_name == BoardNames.ZC702:
             return ZC702()
+        elif self.cfg.board_name == BoardNames.ARTY_200T_CUSTOM_LIDAR:
+            return ARTY_200T_CUSTOM_LIDAR()
         else:
             raise Exception(f'The requested board {self.cfg.board_name} could not be found.')
 
@@ -91,7 +109,7 @@ class VivadoConfig():
         self.parent = parent
 
         # set project name
-        self.project_name = 'project'
+        self.project_name = 'prj'
         # intermediate variables for generic Xilinx path
         if platform in {'win32', 'cygwin'}:
             xilinx_version_path = parent.cfg_dict['TOOLS_xilinx']
@@ -99,10 +117,15 @@ class VivadoConfig():
         # set path to vivado binary
         self.hints = [lambda: os.path.join(env['VIVADO_INSTALL_PATH'], 'bin'),
                       lambda: os.path.join(parent.cfg_dict['INICIO_TOOLS'], xilinx_version_path, "Vivado", xilinx_version, "bin" ),]
-
+        # lsf options for tcl mode of Vivado
+        self.lsf_opts_ls = ''
+        self.lsf_opts = parent.cfg.lsf_opts
         if platform in {'linux', 'linux2'}:
             sorted_dirs = sorted(glob('/tools/Xilinx/Vivado/*.*'), key=vivado_search_key)
             self.hints.extend(lambda: os.path.join(dir_, 'bin') for dir_ in sorted_dirs)
+            if 'CAMINO' in os.environ:
+                self.lsf_opts = "-eh_ram 70000 -eh_ncpu 8 -eh_ui inicio_batch"
+                self.lsf_opts_ls = "-eh_ram 70000 -eh_ncpu 8 -eh_dispatch LS_SHELL"
 
         self._vivado = vivado
         # set various project options
@@ -123,7 +146,7 @@ class XceliumConfig():
     def __init__(self, parent: EmuConfig, xrun=None):
         # save reference to parent config
         self.parent = parent
-
+        self.lsf_opts = parent.cfg.lsf_opts
         # set path to xrun binary
         self.hints = [lambda: os.path.join(env['XCELIUM_INSTALL_PATH'], 'bin')]
         self._xrun = xrun
@@ -131,11 +154,14 @@ class XceliumConfig():
         # name of TCL file
         self.tcl_input = 'input.tcl'
 
+
+
     @property
     def xrun(self):
         if self._xrun is None:
             try:
                 self._xrun = find_tool(name='ifxxcelium', hints=self.hints)
+                self.lsf_opts = "-eh_ncpu 4"
                 #self._xrun += " execute"
             except:
                 self._xrun = find_tool(name='xrun', hints=self.hints)
@@ -185,6 +211,9 @@ class GtkWaveConfig():
                       lambda: os.path.join(parent.cfg_dict['INICIO_TOOLS'], parent.cfg_dict['TOOLS_gtkwave'], 'bin')]
         self._gtkwave = gtkwave
         self.gtkw_config = None
+        self.lsf_opts = parent.cfg.lsf_opts
+        if 'CAMINO' in os.environ:
+            self.lsf_opts = "-eh_ram 7000"
 
     @property
     def gtkwave(self):
@@ -201,6 +230,9 @@ class SimVisionConfig():
         self.hints = [lambda: os.path.join(env['SIMVISION_INSTALL_PATH'], 'bin')]
         self._simvision = simvision
         self.svcf_config = None
+        self.lsf_opts = parent.cfg.lsf_opts
+        if 'CAMINO' in os.environ:
+            self.lsf_opts = "-eh_ram 7000"
 
     @property
     def simvision(self):
@@ -214,19 +246,59 @@ class Config(BaseConfig):
     """
     def __init__(self, cfg_file):
         super().__init__(cfg_file=cfg_file, section=ConfigSections.PROJECT)
+
         self.dec_bits = 24
+        """ type(int) : number of bits used for decimation threshold value, which controls the ila capture feature """
+
         self.board_name = BoardNames.PYNQ_Z1
-        #self.board_name = BoardNames.VC707
+        """ type(str) : name of FPGA board, that shall be used for this project. """
+
         self.emu_clk_freq = 25e6
+        """ type(float) : emulation frequency in Hz, that is used as main independent clk in the design. """
+
         self.preprocess_only = False
+        """ type(bool) : indicate, whether or not only conduct elaboration and compile during simulation run. This will
+            show all Macros expanded and currently only works for icarus verilog. """
+
         self.jtag_freq = 15e6
-        self.fpga_sim_crtl = FPGASimCtrl.VIVADO_VIO
-        self.plugins = []
-        self.plugins.append('msdsl')
+        """ type(float) : jtag communication frequency in Hz. This impacts the frequency used for programming the 
+            bitstream onto the FPGA, a value that is set too high might cause stability issues. """
+
+        self.plugins = ['msdsl']
+        """ type(list(str)) : list of plugins that shall be used in the scope of this project. """
+
         self.dt = 0.1e-6
-        self.dt_exponent = -46
-        self.dt_width = 27
+        """ type(float) : globally used dt value for projects, that strictly use fixed-timestep-based models. """
+
+        self.dt_scale = 1e-15
+        """ type(float) : resolution for dt request and response signals (e.g., "1e-15" means 1fs resolution). """
+
+        self.dt_width = 32
+        """ type(int) : number of bits used for dt request and response signals """
+
         self.time_width = 64
+        """ type(int) : number of bits used for emulation time signal.  TODO: explore issue with using
+            values larger than 39 (related to ILA). """
+        
+        self.lsf_opts = ''
+        """ type(string) : LSF options which can be passed to our tool wrapper commands in the Inicio Flowpackage
+            Execution handler options, e.g. '-eh_ncpu 8' can be used"""
+
+        self.ila_depth = 4096
+        """ type(int) : number of samples that can be stored in the ILA for each signal that is probed. Valid values are:
+            1,024, 2,048, 4,096, 8,192, 16,384, 32,768, 65,536, 131,072. Also when choosing this value keep in mind,
+            that this will consume Block Memory. Possible values are further limited by the number of signals to be 
+            stored, number of block memory cells available on the FPGA board and also block memory that was consumed 
+            already be the design itself. """
+
+        self.cpu_debug_mode = False
+        """ type(bool) : Enables probing all signals for either everything below the testbench, or for modules 
+            provided in the cpu_debug_hierarchies attribute."""
+
+        self.cpu_debug_hierarchies = None
+        """ type(str, list) : Either tuple of depth and absolute path or list of tuples of depth and absolute path to 
+            design modules, for which all signals shall be stored in result file. e.g.:
+            [(0, top.tb_i.filter_i)]"""
 
 def find_tool(name, hints=None, sys_path_hint=True):
     # set defaults
